<?php
if (!defined('_PS_VERSION_')) {
  exit;
}
 
class AdminlienproController extends ModuleAdminController
{
	public function __construct()
    {
        $this->bootstrap = true;
        $this->table = 'order';
        $this->className = 'Order';
        $this->lang = false;
        $this->explicitSelect = true;
		$this->show_toolbar = false;
		$this->allow_export = true;
        $this->deleted = false;

        parent::__construct();
		
        $this->fields_list = array(
            'id_order' => array(
                'title' => $this->trans('ID', array(), 'Admin.Global'),
                'align' => 'text-center',
                'class' => 'fixed-width-xs'
            ),
            'customer' => array(
                'title' => $this->trans('Customer', array(), 'Admin.Global'),
                'havingFilter' => true
            ),
			'address' => array(
                'title' => $this->trans('Delivery', array(), 'Admin.Global'),
                'havingFilter' => true
			),
			'shipping_number' => array(
                'title' => $this->trans('Suivi', array(), 'Admin.Global'),
                'havingFilter' => true
            ),
            'total_paid_tax_incl' => array(
                'title' => $this->trans('Total', array(), 'Admin.Global'),
                'align' => 'text-right',
                'type' => 'price',
                'currency' => true,
                'callback' => 'setOrderCurrency',
                'badge_success' => true
            ),
			 'total_shipping_tax_incl' => array(
                'title' => $this->trans('Course', array(), 'Admin.Global'),
                'align' => 'text-right',
                'type' => 'price',
                'currency' => true,
                'callback' => 'setOrderCurrency',
                'badge_success' => true
            ),
            'date_add' => array(
                'title' => $this->trans('Date', array(), 'Admin.Global'),
                'align' => 'text-right',
                'type' => 'datetime',
                'filter_key' => 'a!date_add'
            )
        );
		
		$this->_select = '
			CONCAT(ad.firstname, \' \', ad.lastname) AS customer,
			CONCAT(ad.address1, \', \', ad.postcode, \', \', ad.city) AS address,';
	
			
        $this->_join = '
		LEFT JOIN `'._DB_PREFIX_.'address` ad ON ad.id_address = a.id_address_delivery
		LEFT JOIN `'._DB_PREFIX_.'carrier` ca ON ca.id_carrier = a.id_carrier
		';
	
		$this->_where ='AND ca.id_carrier = '.Configuration::get('LIENPRO_CARRIER_ID');
	}

	public static function setOrderCurrency($echo, $tr)
    {
        $order = new Order($tr['id_order']);
        return Tools::displayPrice($echo, (int)$order->id_currency);
    }

	
	public function renderForm()
    {
        $this->fields_form = array(
            'legend' => array(
                'title' => $this->trans('Orders', array(), 'Admin.Orderscustomers.Feature'),
                'icon' => 'icon-credit-card'
            ),
            'input' => array(
				array(
					'type' => 'text',
					'label' => $this->trans('Numéro de suivi', array(), 'Admin.Orderscustomers.Feature'),
					'name' => 'shipping_number',
					'required' => false,
					'col' => '4',
					'hint' => $this->trans("Le numéro de suivi attribué à cette commande par l'API, cliquer sur Enregistrer pour en recevoir un automatiquement, il est inutile de remplir ce champ.", array(), 'Admin.Orderscustomers.Feature')
				),
				array(
					'type' => 'hidden',
                    'name' => 'id_order'
				),
				array(
                    'type' => 'hidden',
                    'name' => 'id_address_delivery'
                )
			),
            'submit' => array(
                'title' => $this->trans('Save', array(), 'Admin.Actions'),
            )
        );
			
        return parent::renderForm();
    }

    public function processSave()
    {
        if (Tools::getValue('submitFormAjax')) {
            $this->redirect_after = false;
        }
		
		if(!Tools::getValue('shipping_number') == ''){
			$this->errors[] = $this->trans("Le champ n'est pas vide, si vous avez entré une valeur manuellement, veuillez l'effacer et recommencer l'opération.", array(), 'Admin.Orderscustomers.Notification');
		}
		
		$pickup_address = $this->context->shop->getAddress();		
		$pickup_array = array($pickup_address->address1, $pickup_address->city, (new Country($pickup_address->id_country))->name[$this->context->language->id]);
		$pickup = implode(', ', $pickup_array);
		
		$sql = 'SELECT address1, city FROM '._DB_PREFIX_.'address WHERE id_address = '.Tools::getValue('id_address_delivery')."";
		if(!$delivery_array = Db::getInstance()->ExecuteS($sql))
				$this->errors[] = $this->trans("Requête erronée.", array(), 'Admin.Orderscustomers.Notification');
		$delivery = $delivery_array['0']['address1'].", ".$delivery_array['0']['city'];
		
		$sql = 'SELECT name, latitude, longitude FROM '._DB_PREFIX_.'store';
		if(!$picking_array = Db::getInstance()->ExecuteS($sql))
				$this->errors[] = $this->trans("Requête erronée.", array(), 'Admin.Orderscustomers.Notification');
		
		$sql = 'SELECT company, lastname, firstname, phone FROM '._DB_PREFIX_.'address';
		if(!$contact_array = Db::getInstance()->ExecuteS($sql))
				$this->errors[] = $this->trans("Requête erronée.", array(), 'Admin.Orderscustomers.Notification');
		$contact = $contact_array['0']['company']." (".$contact_array['0']['firstname']." ". $contact_array['0']['lastname']." : ". $contact_array['0']['phone'].")";
		
        $curl = curl_init();

		$params = [
			'picking_contact' => $picking_array['0']['name'],
			'picking_latitude' => $picking_array['0']['latitude'],
			'picking_longitude' => $picking_array['0']['longitude'],
			'delivery_contact' => $contact,
			'formatted_address_pick' => $pickup,
			'formatted_address_delivery' => $delivery,
			'token' => Configuration::get('LIENPRO_TOKEN')
		];

		$params_string = http_build_query($params);

		$opts = [
			CURLOPT_URL => 'http://api.lelien.pro/run/create',
			CURLOPT_POST => true,
			CURLOPT_POSTFIELDS => $params_string,
			CURLOPT_RETURNTRANSFER => true
		];

		curl_setopt_array($curl, $opts);

		$response = json_decode(curl_exec($curl), true);
			
		if(!$response['status'] == 'true'){
			$this->errors[] = $this->trans("Opération échouée, vérifiez que vous avez bien configuré votre module dans 'Modules' -> 'Modules installés' -> 'Le Lien Pro' -> 'CONFIGURER'", array(), 'Admin.Orderscustomers.Notification');
		}
		
		$tracking_check = Db::getInstance()->getValue('SELECT `shipping_number` FROM `'._DB_PREFIX_.'orders` WHERE `id_order` = '.Tools::getValue('id_order'));
		if($tracking_check == '')
		{
			if ($this->action == 'save' &&
				!Tools::getValue('id_order') == '' &&  
				!count($this->errors)) 
			{
				$sql = "UPDATE 	"._DB_PREFIX_."orders, "._DB_PREFIX_."order_carrier 
						SET 	"._DB_PREFIX_."orders.shipping_number = '".$response['tracking']."' ,
								"._DB_PREFIX_."order_carrier.tracking_number ='".$response['tracking']."'
						WHERE 	"._DB_PREFIX_."orders.id_order = ".Tools::getValue('id_order')."
						AND 	"._DB_PREFIX_."order_carrier.id_order = ".Tools::getValue('id_order');
	
				
				if (!Db::getInstance()->Execute($sql))
				{
					$this->errors[] = $this->trans("Une erreur est survenue lors de l'attribution du numéro de suivi à la commande." , array(), 'Admin.Orderscustomers.Notification');
				} 
					return true;
			}
		}
        $this->errors[] = $this->trans("Numéro de suivi déjà attribué.", array(), 'Admin.Orderscustomers.Notification');
    }
}