<?php
if (!defined('_PS_VERSION_'))
{
  exit;
}

class LienPro extends CarrierModule
{
	public  $id_carrier;

	private $_html = '';
	private $_postErrors = array();
	private $_moduleName = 'lienpro';

	public function __construct()
	{
		$this->name = 'lienpro';
		$this->tab = 'shipping_logistics';
		$this->version = '1.0';
		$this->author = 'LeLien.pro';
		$this->limited_countries = array('fr');
		
		parent::__construct ();
		
		$this->displayName = $this->l('LeLien');
		$this->description = $this->l('Service de livraison par coursier');

		if (self::isInstalled($this->name))
		{
			global $cookie;
			$carriers = Carrier::getCarriers($cookie->id_lang, true, false, false, NULL, PS_CARRIERS_AND_CARRIER_MODULES_NEED_RANGE);
			$id_carrier_list = array();
			foreach($carriers as $carrier)
				$id_carrier_list[] .= $carrier['id_carrier'];

			$warning = array();
			if (!in_array((int)(Configuration::get('LIENPRO_CARRIER_ID')),
					   $id_carrier_list)) $warning[] .= $this->l('"LeLien"').' ';
			if (!Configuration::get('LIENPRO_OVERCOST'))
				$warning[] .= $this->l('"Le Lien Pro Overcost"').' ';
			if (count($warning))
		$this->warning .= implode(' , ',$warning).$this->l('must be <br />configured');
		}
	}
	
	public function install()
	{
		
		$id_parent = Tab::getIdFromClassName('AdminParentOrders');
		
		$tab = new Tab();
		// Need a foreach for the language
		$tab->name[$this->context->language->id] = $this->l($this->displayName);
		$tab->class_name = 'AdminLienPro';
		$tab->id_parent = $id_parent; 
		$tab->module = $this->name;
		$tab->add();
		
		$carrierConfig = array(
			'name' => 'Le Lien Pro',
			'id_tax_rules_group' => 0,
			'active' => true, 
			'deleted' => 0,
			'shipping_handling' => false,
			'range_behavior' => 0,
			'delay' =>  array('fr' => 'Livraison en moins de 24 heures !', Language::getIsoById(Configuration::get('PS_LANG_DEFAULT')) => 'Livraison en moins de 24 heures !'),
			'url' => 'https://app.lelien.pro/tracking.html?id=@',
			'id_zone' => 1,
			'is_module' => true,
			'shipping_external' => true,
			'external_module_name' => 'lienpro',
			'need_range' => true
		  );

		$id_carrier = $this->installExternalCarrier($carrierConfig);
		Configuration::updateValue('LIENPRO_CARRIER_ID', (int)$id_carrier);
		if (!parent::install() ||
			!Configuration::updateValue('LIENPRO_OVERCOST', '') ||
			!Configuration::updateValue('LIENPRO_TOKEN', '') ||
			!$this->registerHook('updateCarrier'))
			return false;
		return true;
	}
	
	public function uninstall()
	{
		// Uninstall
		if (!parent::uninstall() ||
		    !Configuration::deleteByName('LIENPRO_OVERCOST') ||
			!Configuration::deleteByName('LIENPRO_TOKEN') ||
		    !$this->unregisterHook('updateCarrier'))
			return false;
		
		//Delete Tab
		$tab = new Tab((int)Tab::getIdFromClassName('AdminLienPro'));
        $tab->delete();
        
		// Delete External Carrier
		$Carrier = new Carrier((int)(Configuration::get('LIENPRO_CARRIER_ID')));
		
		// If external carrier is default set other one as default
		if (Configuration::get('PS_CARRIER_DEFAULT') == (int)($Carrier->id))
		{
			global $cookie;
			$carriersD = Carrier::getCarriers($cookie->id_lang, true, false, false, NULL, PS_CARRIERS_AND_CARRIER_MODULES_NEED_RANGE);
			foreach($carriersD as $carrierD)
				if ($carrierD['active'] AND !$carrierD['deleted'] AND ($carrierD['name'] != $this->_config['name']))
					Configuration::updateValue('PS_CARRIER_DEFAULT', $carrierD['id_carrier']);
		}

		// Then delete Carrier
		$Carrier->deleted = 1;
		if (!$Carrier->update())
			return false;
		
		return true;
	}
	
	public static function installExternalCarrier($config)
	{
		$carrier = new Carrier();
		$carrier->name = $config['name'];
		$carrier->id_tax_rules_group = $config['id_tax_rules_group'];
		$carrier->id_zone = $config['id_zone'];
		$carrier->active = $config['active'];
		$carrier->deleted = $config['deleted'];
		$carrier->delay = $config['delay'];
		$carrier->url = $config['url'];
		$carrier->shipping_handling = $config['shipping_handling'];
		$carrier->range_behavior = $config['range_behavior'];
		$carrier->is_module = $config['is_module'];
		$carrier->shipping_external = $config['shipping_external'];
		$carrier->external_module_name = $config['external_module_name'];
		$carrier->need_range = $config['need_range'];

		$languages = Language::getLanguages(true);
		foreach ($languages as $language)
		{
			if ($language['iso_code'] == 'fr')
				$carrier->delay[(int)$language['id_lang']] = $config['delay'][$language['iso_code']];
			if ($language['iso_code'] == Language::getIsoById(Configuration::get('PS_LANG_DEFAULT')))
				$carrier->delay[(int)$language['id_lang']] = $config['delay'][$language['iso_code']];
		}

		if ($carrier->add())
		{
			$groups = Group::getGroups(true);
			foreach ($groups as $group)
				Db::getInstance()->insert(_DB_PREFIX_.'carrier_group', array('id_carrier' => (int)($carrier->id), 'id_group' => (int)($group['id_group'])), 'INSERT');

			$rangePrice = new RangePrice();
			$rangePrice->id_carrier = $carrier->id;
			$rangePrice->delimiter1 = '0';
			$rangePrice->delimiter2 = '10000';
			$rangePrice->add();

			$rangeWeight = new RangeWeight();
			$rangeWeight->id_carrier = $carrier->id;
			$rangeWeight->delimiter1 = '0';
			$rangeWeight->delimiter2 = '10000';
			$rangeWeight->add();

			$zones = Zone::getZones(true);
			foreach ($zones as $zone)
			{
				Db::getInstance()->insert(_DB_PREFIX_.'carrier_zone', array('id_carrier' => (int)($carrier->id), 'id_zone' => (int)($zone['id_zone'])), 'INSERT');
				Db::getInstance()->update(_DB_PREFIX_.'delivery', array('id_carrier' => (int)($carrier->id), 'id_range_price' => (int)($rangePrice->id), 'id_range_weight' => NULL, 'id_zone' => (int)($zone['id_zone']), 'price' => '0'), 'INSERT');
				Db::getInstance()->update(_DB_PREFIX_.'delivery', array('id_carrier' => (int)($carrier->id), 'id_range_price' => NULL, 'id_range_weight' => (int)($rangeWeight->id), 'id_zone' => (int)($zone['id_zone']), 'price' => '0'), 'INSERT');
			}

			// Copy Logo
			if (!copy(dirname(__FILE__).'/carrier.jpg', _PS_SHIP_IMG_DIR_.'/'.(int)$carrier->id.'.jpg'))	
				return false;

			// Return ID Carrier
			return (int)($carrier->id);
		}

		return false;
	}
	
	public function getContent()
	{
		$this->_html .= '<h2>' . $this->l('Le Lien Pro').'</h2>';
		if (!empty($_POST) AND Tools::isSubmit('submitSave'))
		{
			$this->_postValidation();
			if (!sizeof($this->_postErrors))
				$this->_postProcess();
			else
				foreach ($this->_postErrors AS $err)
					$this->_html .= '<div class="alert error"><img src="'._PS_IMG_.'admin/forbbiden.gif" alt="nok" />&nbsp;'.$err.'</div>';
		}
		$this->_displayForm();
		return $this->_html;
	}

	private function _displayForm()
	{
		$this->_html .= '<fieldset>
		<legend><img src="'.$this->_path.'logo.gif" alt="" /> '.$this->l('Conifguration du module Le Lien Pro').'</legend>';

		$alert = array();
		if ( Tools::getValue('token_api') == '')
			$alert['carrier'] = 1;

		if (!count($alert))
			$this->_html .= '<img src="'._PS_IMG_.'admin/module_install.png" /><strong>'.$this->l('Le module est configuré et en ligne!').'</strong>';
		else
		{
			$this->_html .= '<img src="'._PS_IMG_.'admin/warn2.png" /><strong>'.$this->l("Le module n'est pas encore configuré, veuillez saisir un token valide.").'</strong>';
		}

		$this->_html .= '</fieldset><div class="clear">&nbsp;</div>
			<style>
				#tabList { clear: left; }
				.tabItem { display: block; background: #FFFFF0; border: 1px solid #CCCCCC; padding: 10px; padding-top: 20px; }
			</style>
			<div id="tabList">
				<div class="tabItem">
					<form action="index.php?tab='.Tools::getValue('tab').'&configure='.Tools::getValue('configure').'&token='.Tools::getValue('token').'&tab_module='.Tools::getValue('tab_module').'&module_name='.Tools::getValue('module_name').'&id_tab=1&section=general" method="post" class="form" id="configForm">
						<fieldset style="border: 0px;">
							<h4>'.$this->l('Configuration').' :</h4>
							<label>'.$this->l('Token API').' : </label>
							<div class="margin-form">
								<input type="text" size="20" name="token_api" value="'.Tools::getValue('token_api', Configuration::get('LIENPRO_TOKEN')).'" />
							</div>
							<br /><br />
						</fieldset>				
				<div class="margin-form"><input class="button" name="submitSave" type="submit"></div>
			</form>
		</div>
	</div>';
	}

	private function _postValidation()
	{
		// Check configuration values
		if (Tools::getValue('token_api') == '' )
			$this->_postErrors[]  = $this->l('Veuillez saisir un token.');		
		
		$curl = curl_init();

		$params = [
			'token' => Tools::getValue('token_api')
		];

		$params_string = http_build_query($params);

		$opts = [
			CURLOPT_URL => 'https://api.lelien.pro/token/check',
			CURLOPT_POST => true,
			CURLOPT_POSTFIELDS => $params_string,
			CURLOPT_RETURNTRANSFER => true
		];

		curl_setopt_array($curl, $opts);

		$response = json_decode(curl_exec($curl), true);
		if(!$response['status'] == true)
			$this->_postErrors[]  = $this->l('Veuillez saisir un token valide.');		
	}

	private function _postProcess()
	{
		// Saving new configurations
		if (!Configuration::updateValue('LIENPRO_TOKEN', Tools::getValue('token_api')))
				$this->_html .= $this->displayErrors($this->l('Settings failed'));
			$this->_html .= $this->displayConfirmation($this->l('Settings updated'));	
	}

	public function hookupdateCarrier($params)
	{
		if ((int)($params['id_carrier']) == (int)(Configuration::get('LIENPRO_CARRIER_ID')))
		{
			Configuration::updateValue('LIENPRO_CARRIER_ID', (int)($params['carrier']->id));
		}
	}

	public function getOrderShippingCost($params, $shipping_cost)
	{
		$pickup_address = $this->context->shop->getAddress();		
		$pickup_array = array($pickup_address->address1, $pickup_address->city, (new Country($pickup_address->id_country))->name[$this->context->language->id]);
		$pickup	  = implode(', ', $pickup_array);
		
		$cart = new Cart($this->context->cart->id);
		$delivery_address = new Address((int)$cart->id_address_delivery);
		$delivery = $delivery_address->address1 . ", " . $delivery_address->city . ", " . (new Country($pickup_address->id_country))->name[$this->context->language->id];

		$curl = curl_init();
 
		$params = [
			'formatted_address_pick' => $pickup,
			'formatted_address_delivery' => $delivery,
			'token' => Configuration::get('LIENPRO_TOKEN')
		];

		$params_string = http_build_query($params);

		$opts = [
			CURLOPT_URL => 'https://api.lelien.pro/run/calculate',
			CURLOPT_POST => true,
			CURLOPT_POSTFIELDS => $params_string,
			CURLOPT_RETURNTRANSFER => true
		];

		curl_setopt_array($curl, $opts);

		$response = json_decode(curl_exec($curl), true);				
		
		if($response['status'] == true)
		{
			Configuration::updateValue('LIENPRO_OVERCOST', $response['price']);
			if ($this->id_carrier == (int)(Configuration::get('LIENPRO_CARRIER_ID')) &&	Configuration::get('LIENPRO_OVERCOST') > 1)
			{
				return (float)(Configuration::get('LIENPRO_OVERCOST'));	
			}
		}
		return false;
	}

	public function getOrderShippingCostExternal($params)
	{
		$pickup_address = $this->context->shop->getAddress();		
		$pickup_array = array($pickup_address->address1, $pickup_address->city, (new Country($pickup_address->id_country))->name[$this->context->language->id]);
		$pickup	  = implode(', ', $pickup_array);
		
		$cart = new Cart($this->context->cart->id);
		$delivery_address = new Address((int)$cart->id_address_delivery);
		$delivery = $delivery_address->address1 . ", " . $delivery_address->city . ", " . (new Country($pickup_address->id_country))->name[$this->context->language->id];

		$curl = curl_init();
 
		$params = [
			'formatted_address_pick' => $pickup,
			'formatted_address_delivery' => $delivery,
			'token' => Configuration::get('LIENPRO_TOKEN')
		];

		$params_string = http_build_query($params);

		$opts = [
			CURLOPT_URL => 'https://api.lelien.pro/run/calculate',
			CURLOPT_POST => true,
			CURLOPT_POSTFIELDS => $params_string,
			CURLOPT_RETURNTRANSFER => true
		];

		curl_setopt_array($curl, $opts);

		$response = json_decode(curl_exec($curl), true);				
		
		if($response['status'] == true)
		{
			Configuration::updateValue('LIENPRO_OVERCOST', $response['price']);
			if ($this->id_carrier == (int)(Configuration::get('LIENPRO_CARRIER_ID')) &&	Configuration::get('LIENPRO_OVERCOST') > 1)
			{
				return (float)(Configuration::get('LIENPRO_OVERCOST'));	
			}
		}
		return false;
	}
}